#!/usr/bin/bash -x

function reset() {
    local stripe=$1
    
    ceph osd pool delete swh swh --yes-i-really-really-mean-it
    # 
    ceph config set osd osd_pool_erasure_code_stripe_unit $stripe
    ceph osd erasure-code-profile set --force swh k=4 m=2 stripe_unit=$stripe crush-failure-domain=host
    ceph osd pool create swh erasure swh
}

function workload() {
    local objectsize=$1
    local count=$2
    rados --max-objects $count -O $objectsize --pool swh bench 3000000 write -t 16 --no-cleanup --run-name small
}

function run_remote() {
    local objectsize=$1
    local count=$2
    local stripe=$3
    sleep 60
    reset $stripe
    echo before --------------------------------------------------------------------
    ceph df
    ceph osd df
    workload $objectsize $count
    sleep 60 # wait for stats to reach the monitor
    echo after ---------------------------------------------------------------------
    ceph df
    ceph osd df
}

function run_local() {
    local minalloc=$1
    local stripe=$2
    local objectsize=$3
    local count=$4

    echo "==========================================================================="
    echo "================= min alloc $minalloc, $count x $objectsize bytes objects"
    ./build-vms.sh restart
	if test $minalloc -gt 4096 ; then
	    bdevsize=4096
	else
	    bdevsize=$minalloc
	fi
	sed -i \
	    -e "s/^bluestore min alloc size.*/bluestore min alloc size = $minalloc/" \
	    -e "s/^bdev block size.*/bdev block size = $bdevsize/" \
	    templates/ceph.conf.j2
    ansible-playbook -i inventory ceph.yml
    ansible -i inventory -a 'du -sh /var/ceph/osd' ceph
    scp bench-rados.sh debian@10.11.12.211:/tmp/bench-rados.sh ; ssh debian@10.11.12.211 sudo bash -x /tmp/bench-rados.sh run_remote $objectsize $count $stripe
    ansible -i inventory -a 'du -sh /var/ceph/osd' ceph
}

function main() {
    while read minalloc stripe objectsize count ; do
	run_local $minalloc $stripe $objectsize $count |& tee bench-rados-$minalloc-$stripe-$objectsize-$count.out
    done
}

"$@"
